/**********************************************************************************************
Copyright (c) 2013 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_build_backtest_abt.sas
Function       : This macro creates a modeling ABT and scores it.

Authors        :  CSB Team
Date           :  26JUL2013
SAS            :  9.4
Called-by      :  %CSBMVA_BACKTEST_ABT macro
Calls          :  %BANKFDN_RCSET macro to capture the return code for error handling.
				  %DABT_BUILD_BCK_PLG_ABT_WRAPPER macro to build modeling ABT.
				
Input Datasets :  MODEL_MASTER_EXTENSION (APDM library)
                  MODEL_MASTER (APDM library)
				  MODELING_ABT_MASTER (APDM library)
                  MODEL_X_ACT_OUTCOME_VAR (APDM library)
				  VARIABLE_MASTER (APDM library)
                  LEVEL_MASTER (APDM library)
                  _<MODEL_RK>_BACKTEST_ABT_STATUS (BACKTEST library)

Output Datasets:  _<MODEL_RK>_MDL_BCK (USRDEBUG library)
                  _<MODEL_RK>_SCR_BCK (USRDEBUG library)

Logic          :  1. Use apdm.MODEL_MASTER_EXTENSION table to get the 
                     Project_sk & Model_master_sk
                  2. Using Abt_no, get the name of modeling abt & scored dataset.
                  3. According to the value of abt_dt_cnt, set stacked abt paarameter to yes for 
                     multiple time points.
                  4. Call %DABT_BUILD_BCK_PLG_ABT_WRAPPER macro to build the modeling ABT
                     This ABT is created in DABT workspace. Copy it to usrdebug library.
                  5. Find the metadata Id of model from model_master table.
                  6. Rename the variables appropriatly for actuals & estimated values.
                 

Parameters     :  MODEL_RK  -> Retained Key of the model for which BackTesting Process called.
                  ABT_NO -> ABT no from _<MODEL_RK>_BACKTEST_ABT_STATUS table.
                  ABT_DT_CNT -> no of date values passed.
                  SAS_REPOSITORY -> Name of sas repository.
                  BUILD_STATUS_FLAG -> Name of amacro variable whose value will be set to
                                       show the status of ABT build process.
									   1 - Failure
                                       0 - Sucessful

**********************************************************************************************/

%macro csbmva_build_backtest_abt(model_rk   = ,
								 abt_no     = ,
                                 sas_repository = ,
                                 build_status_flag = ,
                                 abt_dt_cnt=);

*==================================================;
* Defining local macro variables ;
*==================================================;

%local m_project_sk   m_model_master_sk  
       m_mdl_abt_ds   m_scr_abt_ds       m_abt_ch_dt
       path_mdl_abt   m_abt_table_nm     abt_exist_flag
       model_type     mdl_level_sk         m_mdl_id
       m_rep_nm       m_port_path        m_scoring_code_file_nm
	   m_scr_path_csb m_rep_map          scr_status
       m_outcome_var  keep_keys_lst		 abt_run_status
       dsid totalobs rc m_model_src_typ_cd;
%let trans_rc = 0;

*==================================================;
* Getting project_sk & model_master_sk;
*==================================================;

proc sql noprint;
  select a.model_id, model_type_cd, project_sk ,model_sk,c.model_source_type_cd, a.level_sk
    into :m_model_id, :model_type, :m_project_sk, :m_model_master_sk, :m_model_src_typ_cd, :mdl_level_sk
  from &dabt_lib..model_master as a,  &dabt_lib..model_master_extension as b,&dabt_lib..MODEL_SOURCE_TYPE_MASTER as c
  where b.model_key=&model_rk.
  and b.model_key=a.model_sk
  and a.model_source_type_sk = c.model_source_type_sk;
quit;
%bankfdn_rcset(&sqlrc);

%let m_model_id = &m_model_id;
%let model_type = &model_type;
%let model_type = %str(%')&model_type%str(%');

%let m_project_sk = &m_project_sk;
%let m_model_master_sk = &m_model_master_sk;
%let mdl_level_sk = &mdl_level_sk;
%let m_model_src_typ_cd = &m_model_src_typ_cd;

*==================================================;
* Getting required information from;
* BACKTEST_ABT_STATUS table;
*==================================================;

proc sql noprint;
  select mdl_abt_ds, scr_abt_ds, compress(abt_dt) into
    :m_mdl_abt_ds, :m_scr_abt_ds, :m_abt_ch_dt
  from &csb_apdm_libref..build_backtest_abt_status
  where abt_no = &abt_no;
quit;
%bankfdn_rcset(&sqlrc);
%let m_abt_ch_dt = &m_abt_ch_dt;

*==================================================;
* Updating BUILD_BACKTEST_ABT_STATUS run status;
*==================================================;

proc sql;
  update &csb_apdm_libref..build_backtest_abt_status
  set status_cd = 'RUNN'
  where abt_no = &abt_no;
quit;

*=========================================================;
* Identifying if the abt to be  created is stacked or not;
*=========================================================;
%if %eval(&abt_dt_cnt) gt 1 %then %let m_abt_stk_flg_val=Y;
%else %let m_abt_stk_flg_val=N;

*==================================================;
* Creating modeling abt and score it;
*==================================================;

%put SYSCC pre ABT build = &syscc;

/* Staging path location for Backtest ABT */
%let csb_app_user = %scan(&_metauser,1,@);
%let stg_path = &CSB_CONFIG_DIR./AppData/&CSB_CONFIG_FOLDER_NM./&CSB_VERSION./logs/userlog/&CSB_APP_USER./backtesting/&model_rk.;

 %dabt_build_bck_plg_abt_wrapper(m_model_id     	  = &m_model_id , 
								m_mdl_type        	  = BCK,
								m_abt_bld_dt   		  = %nrbquote(&m_abt_ch_dt),
								m_abt_stk_flg     	  = &m_abt_stk_flg_val,
								m_abt_stk_size 		  = 0 ,
								m_replace_abt_flg 	  = Y,
								m_abt_libref          = &DABT_BACKTESTING_ABT_LIBREF,
								m_abt_tbl_nm          = &m_mdl_abt_ds,
								m_perform_scoring_flg = Y,
								m_scored_libref       = &DABT_BACKTESTING_ABT_LIBREF,
								m_scored_ds_nm        = &m_scr_abt_ds,
								m_abt_lst_prc_by_usr  = &SYSUSERID,
								m_abt_build_status    = abt_run_status,
                                m_unique_id_for_scr_tbl=&abt_no,
								m_ref_path_bld        = %quote(&stg_path.),
								m_log_file_nm		  = %quote(backtesting_abt_wrapper.log)
							    );

%put SYSCC post ABT build = &syscc;

/* Redirecting log to stored proc */
proc printto log="&log_name";
run;

%let &build_status_flag = &abt_run_status;

%if &abt_run_status ne 0 %then %do;
  %bankfdn_rcset(999);
  %return;
%end;

/* Copying it in CSB area */
data usrdebug.&m_scr_abt_ds;
  set &DABT_BACKTESTING_ABT_LIBREF..&m_scr_abt_ds;
run;
%bankfdn_rcset(&syserr);

%let dsid=%sysfunc(open(usrdebug.&m_scr_abt_ds));
%let totalobs=%sysfunc(attrn(&dsid,nobs));
%let rc=%sysfunc(close(&dsid));

%if &totalobs > 0 %then %do;
%end;
%else %do;
  %put ERROR: No data;
  %bankfdn_rcset(9999);
  %return;
%end;

*==================================================;
*	Renaming the variables in scored dataset ;
*==================================================;

/* Finding the outcome variable of the model */
proc sql noprint;
  select VARIABLE_COLUMN_NM into :m_outcome_var
  from &dabt_lib..model_x_act_outcome_var as a,
       &dabt_lib..variable_master as b
  where a.variable_sk  = b.variable_sk
  and   a.target_variable_flg = 'Y'
  and   a.model_sk = &m_model_master_sk;
quit;
%bankfdn_rcset(&sqlrc);
%let m_outcome_var = &m_outcome_var;

/* START - renaming variable for actual value */
proc datasets library = usrdebug nodetails nolist;
  modify &m_scr_abt_ds;
    %if %kupcase(&model_type) = &PD_ONG_MODEL. or %kupcase(&model_type) = &PD_APP_MODEL. %then %do;
	  rename &m_outcome_var = ACTUAL_GOOD_BAD;
	%end;
	%else %if %kupcase(&model_type) = &LGD_MODEL. %then %do;
      rename LGD = ACTUAL_LGD EAD = EXPOSURE_AT_DEFAULT;
	%end;
	%else %if %kupcase(&model_type) = &CCF_MODEL. %then %do;
	  rename &m_outcome_var = ACTUAL_CCF;
	%end;
  run;
quit;
%bankfdn_rcset(&syserr);

/* END - renaming variable for actual value */

/* START - renaming variable for ESTIMATED value */

proc datasets library = usrdebug nodetails nolist;
  modify &m_scr_abt_ds;
    %if %kupcase(&model_type) = &PD_ONG_MODEL. or %kupcase(&model_type) = &PD_APP_MODEL. %then %do;
	  rename SCORECARD_POINTS = SCORE_POINTS 
       		%if &m_model_src_typ_cd ne JDG_SCR %then %do;
        		 P_&m_outcome_var.1 = ESTIMATED_PD
       		%end;
          ;
	%end;
	%else %if %kupcase(&model_type) = &LGD_MODEL. %then %do;
      rename P_LGD = ESTIMATED_LGD;
	%end;
	%else %if %kupcase(&model_type) = &CCF_MODEL. %then %do;
	  rename P_&m_outcome_var = ESTIMATED_CCF;
	%end;
  run;
quit;
%bankfdn_rcset(&syserr);

/* END - renaming variable for ESTIMATED value */

*==================================================;
* code to keep required variables ;
*==================================================;

/* Find list of all higher level keys */
proc sql noprint;
  select LEVEL_KEY_COLUMN_NM into :keep_keys_lst separated by ' '
  from &dabt_lib..level_master
  where level_sk in (select assoc_level_sk
  					 from &dabt_lib..level_assoc
					 where level_sk = &mdl_level_sk)
  or level_sk = &mdl_level_sk;
quit;
%bankfdn_rcset(&sqlrc);

%if %kupcase(&model_type) = &PD_ONG_MODEL. or %kupcase(&model_type) = &PD_APP_MODEL. %then %do;
  data usrdebug.&m_scr_abt_ds;
    set usrdebug.&m_scr_abt_ds;
	length ABT_DT 8;
	format ABT_DT date9.;
	%if &m_abt_stk_flg_val=N %then %do;
    		abt_dt = "&m_abt_ch_dt."d;
	%end;
	%else %do;
		abt_dt=datepart(build_dttm);
	%end;
	score_points = round(score_points);

	if actual_good_bad not in ( 1 . ) then actual_good_bad = 0;

	%if &m_model_src_typ_cd ne JDG_SCR %then %do;
    	keep &keep_keys_lst score_points estimated_pd actual_good_bad abt_dt;
    %end;
	%else %do;
		keep &keep_keys_lst score_points actual_good_bad abt_dt;
	%end;
  run;
  %bankfdn_rcset(&syserr);
%end;
%else %if %kupcase(&model_type) = &LGD_MODEL. %then %do;
  data usrdebug.&m_scr_abt_ds;
    set usrdebug.&m_scr_abt_ds;
    length ABT_DT 8;
	format ABT_DT date9.;

  	%if &m_abt_stk_flg_val=N %then %do;
    		abt_dt = "&m_abt_ch_dt."d;
	%end;
	%else %do;
		abt_dt=datepart(build_dttm);
	%end;
	actual_loss = exposure_at_default + recovery_cost - value_at_recovery;

	keep &keep_keys_lst estimated_lgd actual_lgd default_dt actual_loss
         exposure_at_default abt_dt;
  run;
  %bankfdn_rcset(&syserr);
%end;
%else %if %kupcase(&model_type) = &CCF_MODEL. %then %do;
  data usrdebug.&m_scr_abt_ds;
    set usrdebug.&m_scr_abt_ds;
    length ABT_DT 8;
	format ABT_DT date9.;

    %if &m_abt_stk_flg_val=N %then %do;
    abt_dt = "&m_abt_ch_dt."d;
	%end;
	%else %do;
	abt_dt=datepart(build_dttm);
	%end;
	keep &keep_keys_lst estimated_ccf actual_ccf abt_dt;
  run;
  %bankfdn_rcset(&syserr);
%end;

%if &trans_rc > 4 %then %do;
  %let &build_status_flag = 1;
%end;

%mend csbmva_build_backtest_abt;
