/**********************************************************************************************
Copyright (c) 2013 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_get_models_for_poolscheme.sas
Function       : This macro extracts the model details for a pool scheme and 
				 creates output table csbscr.POOL_SCHEME_X_MODEL_INFO

Authors        :  CSB Team
Date           :  28JUN2013
SAS            :  9.4

Called-by      :  ETL job MasterLoopPoolScoreJob.
Calls          :  %BANKFDN_RCSET macro to capture the return code for error handling.

Input Datasets :  pool_scheme (APDM library)
                  time_dim (DIM library)
  		 		  Pool_scheme_x_model  (APDM library)
		          model_x_pool(APDM library)
				  analytical_model_dim (DIM library)
                  time_frequency_master (APDM library)
				  scoring_model (APDM library)
                  model_x_pool_scheme (APDM library)
                  

Output Datasets:  POOL_SCHEME_X_MODEL_INFO (output_ds_lib library)

Logic          :  1. Join input table with pool_scheme to get pool_scheme_sk.
	              2. Get time_sk from time_dim.
                  3. Get model rk associated with this pool_scheme using model_x_pool_scheme.
                     Get all the required details using analytical_model_dim,time_dim,scoring_model,
				     time_frequency_master.

Parameters     :  intable    -> input dataset with library name .
                  wherecl    -> where clause to be applied on input dataset.	
			    output_ds_lib-> library where output dataset will be created.	

**********************************************************************************************/


%macro  csbmva_get_models_for_poolscheme(intable=,wherecl=,output_ds_lib=);

*==================================================================;
* Initialize output_ds_lib  ;
*==================================================================;
%if &output_ds_lib eq %then %let output_ds_lib=csbscr;
%else  %let output_ds_lib=&output_ds_lib;

*==================================================================;
* Get pool_scheme_sk corresponding to pool_scheme_no ;
*==================================================================;
proc sql;
   create table csbscr.POOL_SCHEME_X_MODEL_JOIN_1 as
  	 select
     	 pool_scheme.POOL_SCHEME_SK as POOL_SCHEME_SK ,
     	 b.*	  
   	 from
      &csb_apdm_libref..pool_scheme as a, 
      &intable as b
  	 where
      pool_scheme.POOL_SCHEME_SK = b.POOL_SCHEME_NO
      %if &wherecl ne %then %do;
     	and &wherecl
	 %end;
   ;

%bankfdn_rcset(&sqlrc);
  
*==================================================================;
* Find LOAD_TIME_SK ;
*==================================================================;
   create table csbscr.POOL_SCHEME_X_MODEL_JOIN_2 as
  	 	select distinct
     		 a.TIME_SK as LOAD_TIME_SK label='Time key associated with this pool scoring data load' ,
			 DHMS(SCORING_DT,23,59,59)as SCR_DTTM format=nldatm21.,
			 b.*     
  		from
      		dim.TIME_DIM as a, 
     		 csbscr.POOL_SCHEME_X_MODEL_JOIN_1 as b
   		where
      		b.SCORING_DT >= datepart(a.PERIOD_FIRST_DTTM)
     		 and b.SCORING_DT <= datepart(a.PERIOD_LAST_DTTM)
   ;

%bankfdn_rcset(&sqlrc);

*==================================================================;
* Create output table pool_scheme_x_model_info ;
*==================================================================;

   create table &output_ds_lib..POOL_SCHEME_X_MODEL_INFO(drop=TARGET_TIME_DTTM STATUS) as
   		select 	a.*,b.model_type_cd as model_type,b.model_rk, c.scoring_model_id as scoring_model_id,source_model_master_key as
                  model_key_frm_pooling label='model_rk used during pooling-same as model_key in model_master_extension table' ,
				case 
					when model_type_cd = &LGD_MODEL
						then intnx(d.dttm_interval_expression, scr_dttm, b.target_period_cnt + &CSB_DEF_CPTR_PERIOD_POST_LGD_SCR.,'end')
					else
						intnx(d.dttm_interval_expression, scr_dttm, b.target_period_cnt,'end')
				end as target_time_dttm format=nldatm21., 
				e.time_sk as target_time_sk label='target_time_sk'
   		from csbscr.pool_scheme_x_model_join_2 as a,dim.analytical_model_dim as b, 
			&csb_apdm_libref..scoring_model c,&csb_apdm_libref..time_frequency_master d, dim.time_dim e,
			&csb_apdm_libref..model_x_pool_scheme f
  		where a.pool_scheme_sk=f.pool_scheme_sk
		  and f.current_indicator=1
		  and f.model_rk=b.source_model_master_key
          and a.scr_dttm between b.valid_start_dttm and b.valid_end_dttm
		  and b.source_scoring_model_key = c.scoring_model_sk
		  and b.target_period_time_freq_cd = d.time_frequency_cd
		  and calculated target_time_dttm = e.period_last_dttm
   ;
    
    
quit;

%bankfdn_rcset(&sqlrc);

%mend csbmva_get_models_for_poolscheme;
