/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_insert_measure_val_mip.sas
Function       : This macro insert measure values in SC_VAR_FACT.

Authors        :  CSB Team
Date           :  01Nov2010
SAS            :  9.2

Called-by      :  %csbmva_scr_var_fact_pd_act 
Calls          :  %RCSET macro to capture the return code for error handling.

Input Datasets :  &MEASURE_DS_NM dataset (&MEASURE_DS_LIB library)
                  
Output Datasets:  _<MODEL_RK>_SC_VAR_FACT (CSB FACT library)

Logic          :  1. Create SCR_VAR_FACT dataset if does not exit.
                  2. Insert measure values in SCR_VAR_FACT.

Parameters     :  MODEL_KEY        -> Model_RK of the model.
                  TIME_KEY         -> Score_time_sk of the model.
                  SCR_VAR_FACT_LIB -> Library where the SCR_VAR_FACT dataset is present.
                  SCR_VAR_FACT_DS  -> Name of the SCR_VAR_FACT table.
                  MEASURE_DS_LIB   -> Library where the dataset for storing measure 
                                      values is present.
                  MEASURE_DS_NM    -> Name of the dataset for storing measure values.
                  DEV_CNT          -> Flag to indicate dev or actual sample.
		 ISDEVELOPMENTCYCLE-> Whether model is in development phase or scoring
					(possible values : true, false)

**********************************************************************************************/

%macro csbmva_insert_measure_val_mip(model_key        = ,time_key        = ,
                                     scr_var_fact_lib = ,scr_var_fact_ds = ,
                                     measure_ds_lib   = ,measure_ds_nm   = ,
                                     dev_cnt          = ,isDevelopmentCycle = );

*==================================================;
* Defining local macro variables ;
*==================================================;

%local  m_model_rk  m_level_sk       m_version_no
       	m_time_key  m_measure_sk     m_measure_value 
       	totalobs    period_end_dttm  m_target_time_sk
       	m_var_sk    i;

*==================================================================;
* Finding the model_key from model_master_extension ;
*==================================================================;

%if &isDevelopmentCycle = true %then %do;
	%let mm_ext_key = &model_key;
%end;
%else %do;
	proc sql noprint;
		select source_model_master_key, level_sk
			into :mm_ext_key, :levelsk
		from dim.analytical_model_dim
		where model_rk = &model_key;
	quit;
	%let mm_ext_key = &mm_ext_key;
%end;

*==================================================;
* Creating SC_VAR_FACT structure ;
*==================================================;

%if %sysfunc(exist(&scr_var_fact_lib..&scr_var_fact_ds)) eq 0 %then %do;
	proc sql;
		CREATE TABLE &scr_var_fact_lib..&scr_var_fact_ds(
		      
               MODEL_RK             NUMERIC(8),
               SCORE_TIME_SK        NUMERIC(8),
			   VARIABLE_SK          NUMERIC(8),
               MEASURE_SK           NUMERIC(8),
		       DEV_CNT              NUMERIC(8),
		       MEASURE_VALUE        NUMERIC(8),
		       TARGET_TIME_SK       NUMERIC(8), 
               LEVEL_SK             NUMERIC(8),
			   CLR_MEASURE          NUMERIC(8),
			   VERSION_NO			NUMERIC(8)
			   
		);

		CREATE UNIQUE INDEX XPK_SC_VAR_FACT ON &scr_var_fact_lib..&scr_var_fact_ds
		(
               MODEL_RK,
               SCORE_TIME_SK,
		       VARIABLE_SK,
               MEASURE_SK,
		       DEV_CNT,
			   VERSION_NO
		);
	quit;
	%bankfdn_rcset(&sqlrc);
%end;

*==================================================;
* Convert time_sk to datetime value ;
*==================================================;

data _null_;
  a = put(&time_key,cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;
%bankfdn_rcset(&syserr);

*==================================================;
* Derive VERSION_NO ;
*==================================================;

%let m_version_no = 1;

%if &dev_cnt = 1 %then %do;
	proc sql noprint;
		select coalesce(max(dev_measure_version_no),0) + 1 into :m_version_no
		from &csb_apdm_libref..model_dev_measure_x_spec_map
		where model_rk = &mm_ext_key.;
	quit;
%end;

*==================================================;
* Derive TARGET_TIME_SK ;
*==================================================;

proc sql noprint;
  select model_type_cd, level_sk, target_period_cnt, target_period_time_freq_cd
    into  :m_model_type, :m_level_sk, :m_tgt_prd, :m_tgt_freq_cd
  from %if &isDevelopmentCycle = true %then &dabt_lib..model_master_extension ;
	%else dim.analytical_model_dim; 
  where %if &isDevelopmentCycle  = true %then model_key ; %else model_rk; = &model_key
  %if &isDevelopmentCycle = true %then  ; %else and valid_start_dttm <= &period_end_dttm <= valid_end_dttm;;
quit;

%let m_model_type = &m_model_type;
%let m_model_type = %str(%')&m_model_type%str(%');
%let m_tgt_freq_cd = &m_tgt_freq_cd;

%let target_prd_adj_cnt = 0;
%if &m_model_type eq &LGD_MODEL %then %do;
	%let target_prd_adj_cnt = &CSB_DEF_CPTR_PERIOD_POST_LGD_SCR;
%end;

data _null_;
	scr_dttm = &period_end_dttm.;
	scr_dt = datepart(scr_dttm);
	tgt_dt = intnx("&m_tgt_freq_cd",scr_dt, &m_tgt_prd + &target_prd_adj_cnt , 'end');
    tgt_dttm = dhms(tgt_dt,23,59,59);
    call symput('m_tgt_dttm',tgt_dttm);

 run;

 %let m_target_time_sk = .;

proc sql noprint;
	select time_sk into :m_target_time_sk
	from dim.time_dim
	where PERIOD_FIRST_DTTM <= &m_tgt_dttm <= PERIOD_LAST_DTTM;
quit;

*==================================================;
* Insert data into SC_VAR_FACT ;
*==================================================;

proc sql noprint;
  select count(*) into :totalobs
  from &measure_ds_lib..&measure_ds_nm;
quit;
%bankfdn_rcset(&sqlrc);

%do i = 1 %to &totalobs;

data _null_;
  pointer = &i;
  set &measure_ds_lib..&measure_ds_nm point = pointer;

  call symput('m_model_rk',model_rk);
  call symput('m_time_key',score_time_sk);
  call symput('m_measure_sk',measure_sk);
  call symput('m_var_sk',variable_sk);
  call symput('m_measure_value',measure_value);
  stop;
run;

proc sql;
  insert into &scr_var_fact_lib..&scr_var_fact_ds
    (MEASURE_SK,SCORE_TIME_SK,TARGET_TIME_SK,DEV_CNT,MODEL_RK,MEASURE_VALUE, LEVEL_SK,
     VARIABLE_SK, VERSION_NO)
      values
	(&m_measure_sk,&m_time_key,&m_target_time_sk,&dev_cnt,&m_model_rk,&m_measure_value,&m_level_sk,
     &m_var_sk, &m_version_no);
quit;
%bankfdn_rcset(&sqlrc);

%end;

%mend csbmva_insert_measure_val_mip;
